/*
 *  Copyright 2005 The Apache Software Foundation or its licensors, as applicable.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

/**
 * @author Nikolay A. Kuznetsov
 * @version $Revision: 1.3.2.2 $
 */
package java.util.regex;

/**
 * Represents word boundary, checks current character and previous one if
 * differet types returns true;
 * 
 * @author Nikolay A. Kuznetsov
 * @version $Revision: 1.3.2.2 $
 */
class WordBoundary extends AbstractSet {

    boolean positive;

    public WordBoundary(boolean positive) {
        this.positive = positive;
    }

    public int matches(int stringIndex, CharSequence testString,
            MatchResultImpl matchResult) {
        boolean left;
        boolean right;

        char ch1 = stringIndex >= testString.length() ? ' ' : testString
                .charAt(stringIndex);
        char ch2 = stringIndex == 0 ? ' ' : testString.charAt(stringIndex - 1);

        int leftBound = matchResult.hasTransparentBounds() ? 0 : matchResult
                .getLeftBound();
        left = (ch1 == ' ') || isSpace(ch1, stringIndex, leftBound, testString);
        right = (ch2 == ' ')
                || isSpace(ch2, stringIndex - 1, leftBound, testString);
        return ((left ^ right) ^ positive) ? -1 : next.matches(stringIndex,
                testString, matchResult);
    }

    /**
     * Returns false, because word boundary does not consumes any characters and
     * do not move string index.
     */
    public boolean hasConsumed(MatchResultImpl matchResult) {
        // only checks boundary, do not consumes characters
        return false;
    }

    protected String getName() {
        return "WordBoundary";
    }

    private boolean isSpace(char ch, int index, int leftBound,
            CharSequence testString) {
        if (Character.isLetterOrDigit(ch) || ch == '_')
            return false;
        if (Character.getType(ch) == Character.NON_SPACING_MARK) {
            for (; --index >= leftBound;) {
                ch = testString.charAt(index);
                if (Character.isLetterOrDigit(ch))
                    return false;
                if (Character.getType(ch) != Character.NON_SPACING_MARK)
                    return true;
            }
        }
        return true;
    }
}
